/*
 * Licensed to Crate.io GmbH ("Crate") under one or more contributor
 * license agreements.  See the NOTICE file distributed with this work for
 * additional information regarding copyright ownership.  Crate licenses
 * this file to you under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.  You may
 * obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the
 * License for the specific language governing permissions and limitations
 * under the License.
 *
 * However, if you have executed another commercial license agreement
 * with Crate these terms will supersede the license and you may use the
 * software solely pursuant to the terms of the relevant commercial agreement.
 */

package io.crate.execution.engine.collect.stats;

import io.crate.expression.reference.sys.job.ContextLog;

import java.util.Collections;
import java.util.Iterator;

public class NoopLogSink<T extends ContextLog> implements LogSink<T> {

    @SuppressWarnings("rawtypes")
    private static final NoopLogSink INSTANCE = new NoopLogSink();

    private NoopLogSink() {
    }

    @SuppressWarnings("unchecked")
    public static <T extends ContextLog> NoopLogSink<T> instance() {
        return (NoopLogSink<T>) INSTANCE;
    }

    @Override
    public void add(T item) {
    }

    @Override
    public void addAll(Iterable<T> iterable) {
    }

    @Override
    public void close() {
    }

    @Override
    public Iterator<T> iterator() {
        return Collections.emptyIterator();
    }
}
